package info.aspirecn.rdc.iov.sjjh.servicenode.supplier.bty.action;

import info.aspirecn.iov.sjjh.commons.lang.ChannelTypeHandleResponseObject;
import info.aspirecn.iov.sjjh.commons.lang.Constant;
import info.aspirecn.rdc.aspirecloud.node.except.utils.ErrorUtils;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.bty.service.BTYChannelService;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.bty.utils.SupplierProperties;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.bty_interface.action.BTYChannelAction;
import lombok.extern.slf4j.Slf4j;
import org.apache.tomcat.util.ExceptionUtils;
import org.apache.tomcat.util.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;


/**
 * @author jianggang
 * @Date 2018/10/24 17:31:46
 * @Description 白天源供应商-接口接入
 */

@Slf4j
@RestController
public class BTYChannelActionImpl implements BTYChannelAction {

    @Autowired
    BTYChannelService btyChannelService;
    @Autowired
    SupplierProperties property;
    @Override
    public ChannelTypeHandleResponseObject twoParamIdCard(
            @RequestHeader(name = "channelId", required = true) String channelId,
            @RequestParam(name = "outTime", required = false) Integer outTime,
            @RequestParam(name = "customBody", required = true) String customBody) {
        log.info("白天源身份证而要素请求--->channelId:{}--->outTime:{},--->customBody:{}",channelId,outTime,customBody);
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes)RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        request.setAttribute(Constant.CHANNEL_TYPE_KEY,Constant.CHANNEL_TYPE_SYNC);
        request.setAttribute("x-isCharge",Constant.IS_NOT_CHARGE);
        request.setAttribute("x-responseCode",Constant.CHANNEL_LOG_ERROR_CODE);
        request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE, Constant.LOG_UPSTREAM_DEFAULT_RESPONSE_CODE);
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try{
         responseObject = btyChannelService.sendTwoParamIdCardRequest(
        		request, channelId,outTime,customBody);
        //日志记录通道响应码
        if(responseObject.getCode() != 1){
            request.setAttribute("x-responseCode",Constant.CHANNEL_LOG_ERROR_CODE);
        }else{
        	 if(Constant.CUSTOMER_RETURN_JK.equals(request.getHeader(Constant.CUSTOMER_RETURN_TYPE_PARA))){
             	String code = property.getCode().get(responseObject.getResultCode());
             	if(code!=null){
             		responseObject.setResultCode(Integer.valueOf(code));
             	}
             }
            request.setAttribute("x-responseCode",Constant.CHANNEL_LOG_SUCCESS_CODE);
        }
        //日志记录通道查询参数
        if(customBody != null){
            try {
                request.setAttribute(Constant.CHANNEL_LOG_QUERY,Base64.encodeBase64String(customBody.getBytes("utf-8")));
            } catch (UnsupportedEncodingException e) {
                ExceptionUtils.handleThrowable(e);
                log.error("异常信息：{}",e);
            }
        }

        request.setAttribute("x-isCharge",responseObject.getIsCharge());

        return responseObject;
       }catch(Exception ex){
    	   ErrorUtils.captureException(ex);
    	   log.error("message:{}",ex);
    	   return responseObject;
       }
    }

    @Override
    public ChannelTypeHandleResponseObject checkIdCardWeiZhong(
            @RequestHeader(name = "channelId", required = true) String channelId,
            @RequestParam(name = "outTime", required = false) Integer outTime,
            @RequestParam(name = "customBody", required = true) String customBody) {
        log.info("白天源身份证二要素-微众加密版--->channelId:{}--->outTime:{},--->customBody:{}",channelId,outTime,customBody);
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes)RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        request.setAttribute(Constant.CHANNEL_TYPE_KEY,Constant.CHANNEL_TYPE_SYNC);
        request.setAttribute("x-isCharge",Constant.IS_NOT_CHARGE);
        request.setAttribute("x-responseCode",Constant.CHANNEL_LOG_ERROR_CODE);
        request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE, Constant.LOG_UPSTREAM_DEFAULT_RESPONSE_CODE);
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try{
            responseObject = btyChannelService.checkIdCardWeiZhong(
                    request, channelId,outTime,customBody);
            //日志记录通道响应码
            if(responseObject.getCode() != 1){
                request.setAttribute("x-responseCode",Constant.CHANNEL_LOG_ERROR_CODE);
            }else{
                if(Constant.CUSTOMER_RETURN_JK.equals(request.getHeader(Constant.CUSTOMER_RETURN_TYPE_PARA))){
                    String code = property.getCode().get(responseObject.getResultCode());
                    if(code!=null){
                        responseObject.setResultCode(Integer.valueOf(code));
                    }
                }
                request.setAttribute("x-responseCode",Constant.CHANNEL_LOG_SUCCESS_CODE);
            }
            //日志记录通道查询参数
            if(customBody != null){
                try {
                    request.setAttribute(Constant.CHANNEL_LOG_QUERY,Base64.encodeBase64String(customBody.getBytes("utf-8")));
                } catch (UnsupportedEncodingException e) {
                    ExceptionUtils.handleThrowable(e);
                    log.error("异常信息：{}",e);
                }
            }

            request.setAttribute("x-isCharge",responseObject.getIsCharge());

            return responseObject;
        }catch(Exception ex){
            ErrorUtils.captureException(ex);
            log.error("message:{}",ex);
            return responseObject;
        }
    }

    @Override
    public ChannelTypeHandleResponseObject checkIdCardMD5(
            @RequestHeader(name = "channelId", required = true) String channelId,
            @RequestParam(name = "outTime", required = false) Integer outTime,
            @RequestParam(name = "customBody", required = true) String customBody) {
        log.info("白天源身份证二要素-MD5加密版--->channelId:{}--->outTime:{},--->customBody:{}",channelId,outTime,customBody);
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes)RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        request.setAttribute(Constant.CHANNEL_TYPE_KEY,Constant.CHANNEL_TYPE_SYNC);
        request.setAttribute("x-isCharge",Constant.IS_NOT_CHARGE);
        request.setAttribute("x-responseCode",Constant.CHANNEL_LOG_ERROR_CODE);
        request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE, Constant.LOG_UPSTREAM_DEFAULT_RESPONSE_CODE);
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try{
            responseObject = btyChannelService.checkIdCardMD5AndSHA256(
                    request, channelId,outTime,customBody, "MD5");
            //日志记录通道响应码
            if(responseObject.getCode() != 1){
                request.setAttribute("x-responseCode",Constant.CHANNEL_LOG_ERROR_CODE);
            }else{
                request.setAttribute("x-responseCode",Constant.CHANNEL_LOG_SUCCESS_CODE);
            }
            //日志记录通道查询参数
            if(customBody != null){
                try {
                    request.setAttribute(Constant.CHANNEL_LOG_QUERY,Base64.encodeBase64String(customBody.getBytes("utf-8")));
                } catch (UnsupportedEncodingException e) {
                    ExceptionUtils.handleThrowable(e);
                    log.error("异常信息：{}",e);
                }
            }

            request.setAttribute("x-isCharge",responseObject.getIsCharge());

            return responseObject;
        }catch(Exception ex){
            ErrorUtils.captureException(ex);
            log.error("message:{}",ex);
            return responseObject;
        }
    }

    @Override
    public ChannelTypeHandleResponseObject checkIdCardSHA256(
            @RequestHeader(name = "channelId", required = true) String channelId,
            @RequestParam(name = "outTime", required = false) Integer outTime,
            @RequestParam(name = "customBody", required = true) String customBody) {
        log.info("白天源身份证二要素-SHA256加密版--->channelId:{}--->outTime:{},--->customBody:{}",channelId,outTime,customBody);
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes)RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        request.setAttribute(Constant.CHANNEL_TYPE_KEY,Constant.CHANNEL_TYPE_SYNC);
        request.setAttribute("x-isCharge",Constant.IS_NOT_CHARGE);
        request.setAttribute("x-responseCode",Constant.CHANNEL_LOG_ERROR_CODE);
        request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE, Constant.LOG_UPSTREAM_DEFAULT_RESPONSE_CODE);
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try{
            responseObject = btyChannelService.checkIdCardMD5AndSHA256(
                    request, channelId,outTime,customBody, "SHA256");
            //日志记录通道响应码
            if(responseObject.getCode() != 1){
                request.setAttribute("x-responseCode",Constant.CHANNEL_LOG_ERROR_CODE);
            }else{
                request.setAttribute("x-responseCode",Constant.CHANNEL_LOG_SUCCESS_CODE);
            }
            //日志记录通道查询参数
            if(customBody != null){
                try {
                    request.setAttribute(Constant.CHANNEL_LOG_QUERY,Base64.encodeBase64String(customBody.getBytes("utf-8")));
                } catch (UnsupportedEncodingException e) {
                    ExceptionUtils.handleThrowable(e);
                    log.error("异常信息：{}",e);
                }
            }

            request.setAttribute("x-isCharge",responseObject.getIsCharge());

            return responseObject;
        }catch(Exception ex){
            ErrorUtils.captureException(ex);
            log.error("message:{}",ex);
            return responseObject;
        }
    }
}
